%Copyright (c) 2014, Ali Faisal
%All rights reserved.

%Combination model explained in:
%Faisal et al., 2014 Towards computational cumulative biology by combining models of biological datasets. (http://arxiv.org/abs/1404.0329)

%Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

%1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

%2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

clear all
%Set the project_folder path:
project_folder = 'path_to_source_directory';

%Paths to data directories
data_folder = fullfile(project_folder,'demo_data');
addpath(data_folder);
addpath(fullfile(project_folder,'code','projsplx'));

tol = 1e-32;
lim = 1000;
reg = 20;
eta = 0.01;

query = (1:5)'; %Number of queries in demo data

fprintf('Queries')
for ii = 1:size(query,1) 
  fprintf(' %d ',ii)
  xx = csvread(sprintf('%s/predlik_exp%d.csv', data_folder, query(ii)));
  ww(ii,:) = zeros(1, size(xx,1));
  breakpt(ii) = lim;
        
  w = rand(1,size(xx,1)); %initialize weights
  w(find(w <= 0)) = 0;
  w = w./sum(w);

  [dsets ndocs] = size(xx);
  funval_old = sum(log(w*xx)) -reg*(sum(w.^2)); 
    
  for i = 1:lim
    delta_x = sum(xx'./(repmat((w*xx)',1,dsets))) - 2*reg.*w;
    
    w_new = w + eta*delta_x;
    w_new = projsplx(w_new); %projection onto simplex
    
    funval_new = sum(log(w_new*xx)) - reg*(sum(w_new.^2));

    myfun(i) = funval_old;
    if  (abs(funval_new - funval_old) < tol )
      breakpt(ii) = i;
      w_new = w;
      break;
    end

    funval_old = funval_new;
    w = w_new;	  
  end

  ww(ii,:) = w_new;

end

output_file = sprintf('%s/network.csv',project_folder);
csvwrite(output_file,ww);
fprintf('\nThe optimized weights are stored under %s\n', output_file);
